# demo 使用教程

## 编译运行环境：腾讯云服务器（需要与订阅实例相同地域，才能够访问到 Kafka 服务器的内网地址），安装 Python3，pip3并安装以下依赖：

    pip install flag
    pip install kafka-python
    pip install avro

## 启动消费：

    python main.py --brokers=xxx --topic=xxx --group=xxx --user=xxx --password=xxx --trans2sql=1

# 启动参数说明

| 参数      |       说明                      |
| ----------| -------------------------------|
| brokers   | Kafka 的内网访问地址（域名+端口号） |
| topic     | Kafka topic 名称                |
| group     | Kafka 消费组名称                 |
| user      | Kafka 消费组账号名               |
| password  | Kafka 消费组密码                 |
| trans2sql | 是否将消费数据转换为 SQL 语句打印   |

# 协议文件

## Record中字段说明

| 字段名称               |     类型       |含义                      |
| ---------------       | --------      | ----------               |
| id                    |   long        | 全局递增Id                |
| version               |   int         | 协议版本，当前版本为1       |
| messageType           |   String      | 消息类型，枚举值："INSERT", "UPDATE", "DELETE", "DDL", "BEGIN", "COMMIT", "HEARTBEAT", "CHECKPOINT"|
| fileName              |   String      | 当前record所在的binlog文件名 |
| position              |   String      | 当前record的在binlog中结束的偏移量，格式为 End_log_pos@binlog文件编号。比如，当前record位于文件mysql-bin.000004中，结束偏移量为2196，则其值为"2196@4" |
| safePosition          |   String      | 当前事务在binlog中开始的偏移量，格式同上   |
| timestamp             |   long        | 写入binlog的时间，unix时间戳，秒级        |
| gtid                  |   String      | 当前的gtid，如：c7c98333-6006-11ed-bfc9-b8cef6e1a231:9   |
| transactionId         |   String      | 事务Id，只有commit事件才会生成事务id                 |
| serverId              |   String      | 源库serverId，查看源库server_id参考 SHOW VARIABLES LIKE 'server_id'  |
| threadId              |   String      | 提交当前事务的会话id，参考 SHOW processlist;         |
| sourceType            |   String      | 源库类型，当前版本只有 MySQL                         |
| sourceVersion         |   String      | 源库版本，查看源库版本参考 SELECT version()           |
| schemaName            |   String      | 库名                   |
| tableName             |   String      | 表名                   |
| objectName            |   String      | 格式为：库名.表名        |
| columns               |   List\<Field> | 表中各列的定义          |
| oldColumns            |   List\<Object>| dml执行前该行的数据，如果是insert消息，该数组为null。数组中元素共有12种类型:Integer,Character,Decimal,Float,Timestamp,DateTime,TimestampWithTimeZone,BinaryGeometry,TextGeometry,BinaryObject,TextObject,EmptyObject，详见demo中定义              |
| newColumns            |   List\<Object>| dml执行后该行的数据，如果是delete消息，该数组为null。数组中元素共有12种类型:Integer,Character,Decimal,Float,Timestamp,DateTime,TimestampWithTimeZone,BinaryGeometry,TextGeometry,BinaryObject,TextObject,EmptyObject，详见demo中定义              |
| sql                   |   String      | ddl的sql语句            |
| executionTime         |   int         | ddl执行时长，单位：秒     |
| heartbeatTimestamp    |   long        | 心跳消息的时间戳，秒级。只有heartbeat消息有意义   |
| syncedGtid            |   String      | 已解析gtid集合，格式形如：c7c98333-6006-11ed-bfc9-b8cef6e1a231:1-13 |
| fakeGtid              |   boolean     | 是否为构造的假gtid。如未开启gtid_mode，则dts会构造一个gtid             |
| pkNames               |   List\<String>| 主键字段，只有dml消息可能有值                  | 
| readerTimestamp       |   long        | dts处理这条数据的时间，unix时间戳，毫秒级         |
| tags                  |   Map\<String,String>| QueryEvent中的status_vars，详细参考：https://dev.mysql.com/doc/dev/mysql-server/latest/classbinary__log_1_1Query__event.html       |
| total                 |   int         | 如果消息分片，记录分片总数。当前版本(version=1)无意义，预留扩展      |
| index                 |   int         | 如果消息分片，记录当前分片的索引。当前版本(version=1)无意义，预留扩展 |

## mysql列属性

Field是描述列属性的对象。包含四个属性：

+ name 列名
+ dataTypeNumber 是binlog中记录的数据类型。取值详见：https://dev.mysql.com/doc/internals/en/com-query-response.html
+ isKey 是否主键
+ originalType ddl中定义的类型

## mysql数据类型转换逻辑

在avro协议中，将mysql类型转换成了以下11中格式。

### Integer

对应数据库中的整数类型，如：TINYINT SMALLINT MEDIUMINT INT BIGINT 和 BIT。  
拥有两个属性：precision表示整数占有的字节数；value为整数值。对于BIT类型，precision固定为8。

### Character

对应数据库中的字符类型，如：CHAR VARCHAR TEXT等。

### Float

对应数据库中的浮点类型，如：FLOAT 和 DOUBLE。 precision和scale分别为information_schema.COLUMNS表中的数据。    
注意，如果表或者列是在订阅任务启动之后新增的，则无法获取到precision和scale。

### Decimal

对应数据库中DECIMAL。

### Timestamp

对应数据库中的TIMESTAMP。其中timestamp单位为秒，millis单位为毫秒，范围为[0,999]。

### DateTime

对应数据库中的其他时间类型

### TimestampWithTimeZone

暂未使用

### BinaryGeometry

对应数据库中的geometry及其子类。

### TextGeometry

暂未使用

### BinaryObject

对应数据库中的二进制类型，如BINARY VARBINARY 和 BLOB等。

### TextObject

对应数据库中的文本类型，如ENUM SET 和 JSON。

### EmptyObject

对应NULL
如果源库的**binlog_row_image**变量设置为**MINIMAL**，binlog不会记录未修改列，此时输出null；
如果表中数据为null，此时输出EmptyObject，反序列化后是字符串null。